import pylab as pl
import casadi as ca

pl.close("all")

# -- Plotting function --

def get_radius_from_index(r_c, i):

    r_i = r[i]
    r_fcn = ca.Function("r_fcn", [R], [r_i])

    return r_fcn(r_c)


def plot_circle(x_i, y_i, r, i):

    eps = 1e-8

    r_i = get_radius_from_index(r, i)

    X = pl.linspace(float(x_i-r_i), float(x_i+r_i), 100)

    Y_pos = pl.sqrt(r_i**2 - (X - x_i)**2 + eps)+y_i
    Y_neg = -pl.sqrt(r_i**2 - (X - x_i)**2 + eps)+y_i

    pl.plot(X, Y_pos, color = "steelblue")
    pl.plot(X, Y_neg, color = "steelblue")


def plot_center(x, y, i):

    pl.scatter(float(x), float(y), color = "steelblue", marker = "x")
    pl.text(float(x+0.2), float(y+0.2), "s_" + str(i+1))


def plot_plate(x_val, y_val, R_val):

    pl.figure(figsize=(6,6))

    for i in range(n_p):

        plot_circle(x_val[i], y_val[i], R_val, i)
        plot_center(x_val[i], y_val[i], i)

    pl.xlim([0, a])
    pl.ylim([0, a])

    pl.xlabel("x")
    pl.ylabel("y")

    pl.grid(True)

    pl.savefig("/tmp/plate.eps", format = "eps", bbox_inches = "tight")



# -- Start of computations --

a = 10.0

n_p = 5

x = ca.MX.sym("x", n_p)
y = ca.MX.sym("y", n_p)

R = ca.MX.sym("R", 1)


r = pl.array([R, R, R, 2*R, 2*R])

g = []
g_min = []
g_max = []

for i in range(n_p):

    g.append(x[i] - r[i])
    g_min.append(0)
    g_max.append(pl.inf)

    g.append(x[i] + r[i])
    g_min.append(-pl.inf)
    g_max.append(a)

    g.append(y[i] - r[i])
    g_min.append(0)
    g_max.append(pl.inf)

    g.append(y[i] + r[i])
    g_min.append(-pl.inf)
    g_max.append(a)

for i in range(n_p):

    for j in range(i+1, n_p):

        g.append((x[i] - x[j])**2 + (y[i] - y[j])**2 - (r[i] + r[j])**2)
        g_min.append(0)
        g_max.append(pl.inf)

g = ca.vertcat(*g)

x_init = pl.array([1, 3, 8, 7, 3])
y_init = pl.array([1, 3, 8, 3, 7])
R_init = 1.0


plot_plate(x_init, y_init, R_init)


V = ca.vertcat(x, y, R)

nlp = {"x": V, "f": -R, "g": g}

solver = ca.nlpsol("solver", "ipopt", nlp)

V_init = ca.vertcat(x_init, y_init, R_init)

x_min = pl.zeros(n_p)
x_max = a * pl.ones(n_p)

y_min = pl.zeros(n_p)
y_max = a * pl.ones(n_p)

R_min = 0.0
R_max = pl.inf

V_min = ca.vertcat(x_min, y_min, R_min)
V_max = ca.vertcat(x_max, y_max, R_max)

solution = solver(x0 = V_init, lbx = V_min, ubx = V_max, \
    lbg = g_min, ubg = g_max)

V_opt = solution["x"]

x_opt = V_opt[:n_p]
y_opt = V_opt[n_p:-1]
R_opt = V_opt[-1]

plot_plate(x_opt, y_opt, R_opt)
